@@ -1,5 +1,4 @@
 #!perl -w
-# -*- coding: utf-8; -*-
 
 use strict;
 use warnings;
@@ -26,16 +25,7 @@ Crypt::OpenSSL::CA::Inline::C - A bag of XS and L<Inline::C> tricks
 
   C_CODE_SAMPLE
 
-  # Then maybe some Perl...
-
-  use Crypt::OpenSSL::CA::Inline::C <<"MORE_C_CODE";
-
-  static
-  void another() {
-    // ...
-  }
-
-  MORE_C_CODE
+  # Lots and lots of the same...
 
   use Crypt::OpenSSL::CA::Inline::C "__END__";
 
@@ -43,10 +33,6 @@ Crypt::OpenSSL::CA::Inline::C - A bag of XS and L<Inline::C> tricks
 
 =head1 DESCRIPTION
 
-B<This documentation is only useful for people who want to hack
-I<Crypt::OpenSSL::CA>. It is of no interest for people who just want
-to use the module.>
-
 This package provides L<Inline::C> goodness to L<Crypt::OpenSSL::CA>
 during development, plus a few tricks of our own.  The idiom in
 L</SYNOPSIS>, used throughout the source code of
@@ -66,9 +52,9 @@ its line, as demonstrated.
 
 =item the "__END__" pragma
 
-The code in L<Crypt::OpenSSL::CA> must use the following pragma to
-signal that it won't attempt to add any L<Inline::C> code after this
-point:
+In order to accomodate hypothetical future extensions, the code in
+L<Crypt::OpenSSL::CA> should use the following pragma to signal that
+it won't attempt to add any L<Inline::C> code after this point:
 
    use Crypt::OpenSSL::CA::Inline::C "__END__";
 
@@ -81,35 +67,24 @@ in L<Inline::C>, L<perlxstut>, L<perlguts> and L<perlapi>, C code that
 compiles itself through I<Crypt::OpenSSL::CA::Inline::C> has access to
 the following C functions:
 
-=head3 char0_value
+=over
 
-  static inline char* char0_value(SV* string);
+=item I<static inline char* char0_value(SV* string)>
 
-Returns the string value of a Perl SV, making sure that it exists and is
-zero-terminated beforehand. If C<string> is undef, returns the empty string
-(B<not> NULL; see L</char0_value_or_null>). If C<string> is tainted, that makes
-no difference; return it, or the empty string, just the same. See
-L<perlguts/Working with SVs>, look for the word "Nevertheless" - I assume
-there is a macro in Perl's convenience stuff that does exactly that already, but
-I don't know it...
+Returns the string value of a Perl SV, making sure that it exists and
+is zero-terminated beforehand. See L<perlguts/Working with SVs>, look
+for the word "Nevertheless". I'm pretty sure there is a macro in
+Perl's convenience stuff that does exactly that already, but I don't
+know it...
 
-=head3 char0_value_or_null
-
-  static inline char* char0_value_or_null(SV* string);
-
-Like L</char0_value>, except that NULL is returned if C<string> is
-undef.
-
-=head3 perl_wrap
-
-   static inline SV* perl_wrap(class, pointer);
+=item I<static inline SV* perl_wrap(class, pointer)>
 
 Creates read-only SV containing the integral value of C<pointer>,
 blesses it into class C<class> and returns it as a SV*.  The return
 value is an adequate Perl wrapper to stand for C<pointer>, as
 demonstrated in L<Inline::C-Cookbook/Object Oriented Inline>.
 
-=head3 perl_unwrap (class, typename, SV*)
+=item I<perl_unwrap(class, typename, SV*)>
 
 The reverse of L</perl_wrap>.  Given a L</perl_wrap>ped SV*, asserts
 that it actually contains an object blessed in class C<class> (lest it
@@ -117,9 +92,7 @@ C<croak>s), extracts the pointer within same, casts it into
 C<typename> and returns it.  This is a macro instead of a static
 inline, so as to be able to perform the polymorphic cast.
 
-=head3 openssl_string_to_SV
-
-  static inline SV* openssl_string_to_SV(char* string);
+=item I<static inline SV* openssl_string_to_SV(char* string)>
 
 Copies over C<string> to a newly-allocated C<SV*> Perl scalar, and
 then frees C<string> using C<OPENSSL_free()>.  Used to transfer
@@ -134,18 +107,14 @@ Check the OpenSSL documentation carefully, and make use of
 L<Crypt::OpenSSL::CA::Test/leaks_bytes_ok> to ascertain experimentally
 that your code doesn't leak memory.
 
-=head3 openssl_buf_to_SV
-
-   static inline SV* openssl_buf_to_SV(char* string, int length);
+=item I<static inline SV* openssl_buf_to_SV(char* string, int length)>
 
 Like L</openssl_string_to_SV> except that the length is specified,
 which allows for C<string> to not contain null characters or not be
 zero-terminated.  Use this form e.g. for ASN.1 buffers returned by
 C<i2d_foobar> OpenSSL functions.
 
-=head3 BIO_mem_to_SV
-
-   static inline SV* BIO_mem_to_SV(BIO *bio);
+=item I<static inline SV* BIO_mem_to_SV(BIO *bio)>
 
 This inline function is intended to be used to return scalar values
 (e.g. PEM strings and RSA moduli) constructed by OpenSSL.  Should be
@@ -159,9 +128,15 @@ C<croak()>s trying (hence the requirement not to have any outstanding
 memory resources allocated in the caller).  Regardless of the outcome,
 C<bio> will be C<BIO_free>()d.
 
-=head2 sslcroak
+=item I<static void ensure_openssl_stuff_loaded()>
 
-   static void sslcroak(char *format, ...);
+Ensures that various stuff is loaded inside OpenSSL, such as
+C<ERR_load_crypto_strings()>, C<OpenSSL_add_all_digests()> and all
+that jazz.  After this function returns,
+C<$Crypt::OpenSSL::CA::openssl_stuff_loaded> will be 1.  Calling it
+several times has no effect.
+
+=item I<static void sslcroak(char *format, ...)>
 
 Like L<perlapi/croak>, except that a blessed exception of class
 I<Crypt::OpenSSL::CA::Error> is generated.  The OpenSSL error
@@ -172,15 +147,14 @@ Note to I<Crypt::OpenSSL::CA> hackers: please select the appropriate
 routine between I<sslcroak> and I<croak>, depending on whether the
 current error condition is being caused by OpenSSL or not; in this way
 callers are able to discriminate errors.  Also, don't be fooled into
-thinking that C<croak>-style error management acts in the same way in
-C and Perl! Because calling C<sslcroak> (or, for that matter,
-L<perlapi/croak>) will return control directly to Perl without running
-any C code, any and all temporary variables that have been allocated
-from C will fail to be de-allocated, thereby causing a memory leak.
+thinking that C<sslcroak> (or, for that matter, C<croak>) is the same
+thing in C and in Perl! Because calling C<sslcroak> will return
+control directly to Perl without running any C code, any and all
+temporary variables that have been allocated from C will fail to be
+de-allocated, thereby causing a memory leak.
 
-Internally, I<sslcroak> works by invoking
-L<Crypt::OpenSSL::CA/_sslcroak_callback> several times, using a rough
-equivalent of the following pseudo-code:
+Internally, I<sslcroak> works by invoking L</_ssl_croak_callback>
+several times, using a rough equivalent of the following pseudo-code:
 
   _sslcroak_callback("-message", $formattedstring);
   _sslcroak_callback("-openssl", $openssl_errorstring_1);
@@ -192,59 +166,7 @@ where $formattedstring is the C<sprintf>-formatted version of the
 arguments passed to I<sslcroak>, and the OpenSSL error strings are
 retrieved using B<ERR_get_error(3)> and B<ERR_error_string(3)>.
 
-=head3 parse_RFC3280_time
-
-  static ASN1_TIME* parse_RFC3280_time(char* datetime,
-                   char** errmsg, char* sslerrmsg);
-
-Parses C<datetime>, a date in "Zulu" format (that is, yyyymmddhhmmssZ,
-with a literal Z at the end), and returns a newly-allocated ASN1_TIME*
-structure utilizing a C<utcTime> encoding for dates in the year 2049
-or before and C<generalizedTime> for dates in 2050 and after.  RFC3280
-dictates that this convention should apply to most date-related fields
-in X509 certificates and CRLs (as per sections 4.1.2.5 for certificate
-validity periods, and 5.1.2.4 through 5.1.2.6 for CRL validity periods
-and certificate revocation times).  By contrast, the C<invalidityDate>
-CRL revocation reason extension is always in C<generalizedTime> and
-this function should not be used there.
-
-If there is an error, NULL is returned, and one (and only
-one) of *errmsg and *sslerrmsg is set to an error string, provided
-that they are not NULL.  Caller should thereafter call I<croak> or
-L</sslcroak> respectively.
-
-=head3 parse_RFC3280_time_or_croak
-
-  static ASN1_TIME* parse_RFC3280_time_or_croak(char* datetime);
-
-Like L</parse_RFC3280_time> except that it handles its errors itself and
-will therefore never return NULL.  The caller should not have an
-outstanding temporary variable that must be freed before it returns,
-or a memory leak will be created; if this is the case, use the more
-clunky L</parse_RFC3280_time> form instead.
-
-=head3 parse_serial
-
-  static ASN1_INTEGER* parse_serial
-              (char* hexserial, char** errmsg, char** sslerrmsg);
-
-Parses hexserial, a lowercase, hexadecimal string that starts with
-"0x", and returns it as a newly-allocated C<ASN1_INTEGER> structure
-that must be freed by caller (with C<ASN1_INTEGER_free>) when done
-with it.  If there is an error, NULL is returned, and one (and only
-one) of *errmsg and *sslerrmsg is set to an error string, provided
-that they are not NULL.  Caller should thereafter call I<croak> or
-L</sslcroak> respectively.
-
-=head3 parse_serial_or_croak
-
-  static ASN1_INTEGER* parse_serial_or_croak(char* hexserial);
-
-Like L</parse_serial> except that it handles its errors itself and
-will therefore never return NULL.  The caller should not have an
-outstanding temporary variable that must be freed before it returns,
-or a memory leak will be created; if this is the case, use the more
-clunky L</parse_serial> form instead.
+=back
 
 =cut
 
@@ -258,27 +180,21 @@ sub _c_boilerplate { <<'C_BOILERPLATE'; }
 #include <openssl/bio.h>    /* Also for BIO_mem_to_SV() */
 
 #include <openssl/opensslv.h>
-#if OPENSSL_VERSION_NUMBER < 0x00907000
-#error OpenSSL version 0.9.7 or later is required. See comments in CA.pm
+#if OPENSSL_VERSION_NUMBER < 0x00905000
+#error OpenSSL version 0.9.5 is required in order to handle Unicode DNs.
 #endif
 
-static inline char* char0_value_or_null(SV* perlscalar) {
-     if (! SvOK(perlscalar)) { return NULL; }
+static inline char* char0_value(SV* perlscalar) {
+     unsigned int length; char* retval;
 
-     STRLEN length;
      SvPV(perlscalar, length);
+     if (! SvPOK(perlscalar)) { return ""; } // Undef
      SvGROW(perlscalar, length + 1);
-
-     char* retval = SvPV_nolen(perlscalar);
+     retval = SvPV_nolen(perlscalar);
      retval[length] = '\0';
      return retval;
 }
 
-static inline char* char0_value(SV* perlscalar) {
-     char* retval = char0_value_or_null(perlscalar);
-     return ( retval ? retval : "" );
-}
-
 static inline SV* perl_wrap(const char* class, void* pointer) {
      SV*      obj = sv_setref_pv(newSV(0), class, pointer);
      if (! obj) { croak("not enough memory"); }
@@ -296,7 +212,7 @@ static inline void* __perl_unwrap(const char* file, int line,
                 "Perl argument (expected an object blessed "
                 "in class ``%s'')", file, line, (class));
     }
-    return (void *)(intptr_t)SvIV(SvRV(obj));
+    return (void *)SvIV(SvRV(obj));
 }
 
 static inline SV* openssl_buf_to_SV(char* string, int length) {
@@ -332,6 +248,17 @@ static inline SV* BIO_mem_to_SV(BIO *mem) {
    return retval;
 }
 
+static inline void ensure_openssl_stuff_loaded() {
+    SV* already_loaded = get_sv
+      ("Crypt::OpenSSL::CA::openssl_stuff_loaded", 1);
+    if (SvOK(already_loaded)) { return; }
+    sv_setiv(already_loaded, 1);
+
+    ERR_load_crypto_strings();
+    OpenSSL_add_all_digests();
+    OpenSSL_add_all_algorithms();
+}
+
 #define ERRBUFSZ 512
 #define THISPACKAGE "Crypt::OpenSSL::CA"
 static void sslcroak(char *fmt, ...) {
@@ -350,6 +277,8 @@ static void sslcroak(char *fmt, ...) {
     unsigned long sslerr;       /* Will iterate through the OpenSSL
                                    error stack */
 
+    ensure_openssl_stuff_loaded(); /* For error strings */
+
     va_start(ap, fmt);
     vsnprintf(croakbuf, ERRBUFSZ, fmt, ap);
     croakbuf[ERRBUFSZ - 1] = '\0';
@@ -377,144 +306,54 @@ static void sslcroak(char *fmt, ...) {
          croak(Nullch);
     } else {
          // Something went bang, revert to the croakbuf.
-         croak("%s", croakbuf);
+         croak(croakbuf);
     }
 }
 
-/* RFC3280, section 4.1.2.5 */
-#define RFC3280_cutoff_date "20500000" "000000"
-static ASN1_TIME* parse_RFC3280_time(char* date,
-                                     char** errmsg, char** sslerrmsg) {
-    int status;
-    int is_generalizedtime;
-    ASN1_TIME* retval;
-
-    if (strlen(date) != strlen(RFC3280_cutoff_date) + 1) {
-         if (errmsg) { *errmsg = "Wrong date length"; }
-         return NULL;
-    }
-    if (date[strlen(RFC3280_cutoff_date)] != 'Z') {
-         if (errmsg) { *errmsg = "Wrong date format"; }
-         return NULL;
-    }
-
-    if (! (retval = ASN1_TIME_new())) {
-         if (errmsg) { *errmsg = "ASN1_TIME_new failed"; }
-         return NULL;
-    }
-
-    is_generalizedtime = (strcmp(date, RFC3280_cutoff_date) > 0);
-    if (! (is_generalizedtime ?
-           ASN1_GENERALIZEDTIME_set_string(retval, date) :
-           ASN1_UTCTIME_set_string(retval, date + 2)) ) {
-        ASN1_TIME_free(retval);
-        if (errmsg) {
-            *errmsg = (is_generalizedtime ?
-               "ASN1_GENERALIZEDTIME_set_string failed (bad date format?)" :
-               "ASN1_UTCTIME_set_string failed (bad date format?)");
-        }
-        return NULL;
-    }
-    return retval;
-}
-
-static ASN1_TIME* parse_RFC3280_time_or_croak(char* date) {
-    char* plainerr = NULL; char* sslerr = NULL;
-    ASN1_INTEGER* retval = NULL;
-    if ((retval = parse_RFC3280_time(date, &plainerr, &sslerr))) {
-        return retval;
-    }
-    if (plainerr) { croak("%s", plainerr); }
-    if (sslerr) { sslcroak("%s", sslerr); }
-    croak("Unknown error in parse_RFC3280_time");
-    return NULL; /* Not reached */
-}
-
-static ASN1_INTEGER* parse_serial(char* hexserial,
-          char** errmsg, char** sslerrmsg) {
-    BIGNUM* serial = NULL;
-    ASN1_INTEGER* retval;
-
-    if (! (hexserial[0] == '0' && hexserial[1] == 'x')) {
-        if (errmsg) {
-            *errmsg = "Bad serial string, should start with 0x";
-        }
-        return NULL;
-    }
-    if (! BN_hex2bn(&serial, hexserial + 2)) {
-        if (sslerrmsg) { *sslerrmsg = "BN_hex2bn failed"; }
-        return NULL;
-    }
-    retval = BN_to_ASN1_INTEGER(serial, NULL);
-    BN_free(serial);
-    if (! retval) {
-        if (sslerrmsg) { *sslerrmsg = "BN_to_ASN1_INTEGER failed"; }
-        return NULL;
-    }
-    return retval;
-}
-
-static ASN1_INTEGER* parse_serial_or_croak(char* hexserial) {
-    char* plainerr = NULL; char* sslerr = NULL;
-    ASN1_INTEGER* retval = NULL;
-    if ((retval = parse_serial(hexserial, &plainerr, &sslerr))) {
-        return retval;
-    }
-    if (plainerr) { croak("%s", plainerr); }
-    if (sslerr) { sslcroak("%s", sslerr); }
-    croak("Unknown error in parse_serial");
-    return NULL; /* Not reached */
-}
-
 C_BOILERPLATE
 
-=head2 BOOT-time effect
-
-Each C<.so> XS module will be fitted with a C<BOOT> section (see
-L<Inline::C/BOOT> which automatically gets executed upon loading it
-with L<DynaLoader> or L<XSLoader>. The C<BOOT> section is the same for
-all subpackages in L<Crypt::OpenSSL::CA>; it ensures that various
-stuff is loaded inside OpenSSL, such as C<ERR_load_crypto_strings()>,
-C<OpenSSL_add_all_algorithms()> and all that jazz.  After the boot
-code completes, C<$Crypt::OpenSSL::CA::openssl_stuff_loaded> will be
-1, so that the following XS modules can skip that when they in turn
-get loaded.
-
-=cut
-
-sub _c_boot_section { <<"ENSURE_OPENSSL_STUFF_LOADED" }
-    SV* already_loaded = get_sv
-      ("Crypt::OpenSSL::CA::openssl_stuff_loaded", 1);
-    if (SvOK(already_loaded)) { return; }
-    sv_setiv(already_loaded, 1);
-
-    ERR_load_crypto_strings();
-    OpenSSL_add_all_algorithms();
-ENSURE_OPENSSL_STUFF_LOADED
-
 =head1 INTERNALS
 
 The C<< use Crypt::OpenSSL::CA::Inline::C >> idiom described in
 L</SYNOPSIS> is implemented in terms of L<Inline>.
 
-=head3 %c_code
+DOCUMENTME
 
-A lexical variable that L</import> uses to accumulate all the C code
-submitted by L<Crypt::OpenSSL::CA>.  Keys are package names, and
-values are snippets of C.
+=over
 
 =cut
 
-my %c_code;
+use Sub::Uplevel;
+BEGIN { die <<"MESSAGE" if $INC{"Inline.pm"}; }
+Sorry, Inline is already loaded and Sub::Uplevel will not be able to
+fool it.  Too bad, it means that Crypt::OpenSSL::CA::Inline::C will
+not work!
+MESSAGE
 
 use Inline::C ();
 
-=head3 import ()
+my $debugging; BEGIN { $debugging = 1; } # FIXME: make this configurable
+
+=item I<import()>
 
 Called whenever one of the C<< use Crypt::OpenSSL::CA::Inline::C "foo"
 >> pragmas (listed in L</SYNOPSIS>) is seen by Perl; performs the
-actual magic of the module.  Stashes everything into L</%c_code>, and
-invokes L</compile_everything> at the end.
+actual magic of the module.  Mostly delegates to L<Inline/import>,
+with the following tweaks:
+
+=over
+
+=item *
+
+L<Inline::C> is configured to compile with all warnings turned into
+errors (i.e. C<-Wall -Werror>) and to link with the OpenSSL libraries;
+
+=item *
+
+the C code that implements the L</Standard Library> is prepended to
+the caller-provided C code.
+
+=back
 
 =cut
 
@@ -525,103 +364,31 @@ sub import {
 
     return $class->compile_everything if ($c_code eq "__END__");
 
-    my ($package, $file, $line) = caller;
-    $c_code{$package} .= sprintf(qq'#line %d "%s"\n', $line + 1, $file)
-        . $c_code;
+    my ($package) = caller;
+
+    uplevel 1, \&Inline::import,
+        (Inline => C => Config =>
+         NAME => $package,
+         ( $Crypt::OpenSSL::CA::VERSION ?
+           (VERSION => $Crypt::OpenSSL::CA::VERSION) : () ),
+         LIBS => join(" ", qw(-lcrypto -lssl),
+                      ($debugging ? qw(-g) : ())),
+         CCFLAGS => join(" ", qw(-Wall -Wno-unused -Werror),
+                         ($debugging ? qw(-g) : ())),
+         ( $debugging ? (CLEAN_AFTER_BUILD => 0) : () ));
+    uplevel 1, \&Inline::import, Inline => C => (_c_boilerplate . $c_code);
     return;
 }
 
-=head3 compile_everything ()
+=item I<compile_everything()>
 
-Called when L</the "__END__" pragma> is seen.  Invokes
-L<compile_namespace> once for every package (that is, every key in
-L</%c_code>), prepending L<_c_boilerplate> each time.
+Called when L</the "__END__" pragma> is seen.  Currently does nothing.
 
 =cut
 
-sub compile_everything {
-    my ($class) = @_;
-    foreach my $package (keys %c_code) {
-        $class->compile_into($package, _c_boilerplate . $c_code{$package},
-                             -boot_section => _c_boot_section());
-    }
-}
+sub compile_everything {}
 
-=head3 compile_into ($package, $c_code, %named_options)
-
-Compile $c_code and make its functions available as part of $package's
-namespace, courtesy to L<Inline::C> magic.  Works by invoking
-L<Inline/import> in a tweaked fashion, so as to compile with all
-warnings turned into errors (i.e. C<-Wall -Werror>) and to link with
-the OpenSSL libraries.  The environment variables are taken into
-account (see L</ENVIRONMENT VARIABLES>).
-
-Available named options are:
-
-=over
-
-=item B<< -boot_section => $c_code >>
-
-Adds $c_code to the BOOT section of the generated .so module.
-
-=back
-
-=cut
-
-sub compile_into {
-    my ($class, $package, $c_code, %opts) = @_;
-    my $compile_params = ($class->full_debugging ?
-                          <<'COMPILE_PARAMS_DEBUG' :
-    CCFLAGS => "-Wall -Wno-unused -Werror -save-temps",
-    OPTIMIZE => "-g",
-    CLEAN_AFTER_BUILD => 0,
-COMPILE_PARAMS_DEBUG
-                          <<'COMPILE_PARAMS_OPTIMIZED');
-    OPTIMIZE => "-g -O2",
-COMPILE_PARAMS_OPTIMIZED
-
-      my $openssl_params = sprintf('LIBS => "%s -lcrypto -lssl",',
-                                   ($ENV{BUILD_OPENSSL_LDFLAGS} or ""));
-    if ($ENV{BUILD_OPENSSL_CFLAGS}) {
-        $openssl_params .= qq' INC => "$ENV{BUILD_OPENSSL_CFLAGS}",';
-    }
-
-    my $version_params =
-      ( $Crypt::OpenSSL::CA::VERSION ?
-        qq'VERSION => "$Crypt::OpenSSL::CA::VERSION",' : "" );
-
-    my $boot_params = ($opts{-boot_section} ? <<"BOOT_CONFIG" : "");
-    BOOT => <<'BOOT_SECTION',
-$opts{-boot_section}
-BOOT_SECTION
-BOOT_CONFIG
-
-    eval <<"FAKE_Inline_C_INVOCATION"; die $@ if $@;
-package $package;
-use Inline C => Config =>
-    NAME => '$package',
-$compile_params
-    $version_params
-    $openssl_params
-    $boot_params
-;
-use Inline C => <<'C_CODE';
-$c_code
-C_CODE
-FAKE_Inline_C_INVOCATION
-
-    return 1;
-}
-
-=head3 full_debugging
-
-Returns true iff the environment variable L</FULL_DEBUGGING> is set.
-
-=cut
-
-sub full_debugging { ! ! $ENV{FULL_DEBUGGING} }
-
-=head3 installed_version
+=item I<installed_version>
 
 Returns what the source code of this module will look like (with POD
 and everything) after it is installed.  The installed version is a dud
@@ -634,13 +401,13 @@ compile time.
 
 =cut
 
-sub installed_version { <<'INSTALLED_VERSION' }
+sub installed_version { <<'INSTALLED_VERSION'; }
 #!perl -w
 
 package Crypt::OpenSSL::CA::Inline::C;
 
 use strict;
-use XSLoader;
+require DynaLoader;
 
 sub import {
     my ($class, $stuff) = @_;
@@ -649,8 +416,8 @@ sub import {
 
     my ($package) = caller;
     no strict "refs";
-    push @{$package."::ISA"}, qw(XSLoader);
-    { no warnings "redefine"; XSLoader::load($package); }
+    push @{$package."::ISA"}, qw(DynaLoader);
+    $package->bootstrap;
 }
 
 =head1 NAME
@@ -668,8 +435,8 @@ Crypt::OpenSSL::CA
 
 =head1 DESCRIPTION
 
-This package simply loads the DLLs that contain the parts of
-L<Crypt::OpenSSL::CA> that are made of XS code. It is a stubbed-down
+This package simply loads the DLL that contains the part of
+L<Crypt::OpenSSL::CA> that is made of XS code. It is a stubbed-down
 version of the full-fledged I<Crypt::OpenSSL::CA::Inline::C> that
 replaces the real thing at module install time.
 
@@ -687,36 +454,14 @@ INSTALLED_VERSION
 
 =end this_pod_is_not_ours
 
-=head1 ENVIRONMENT VARIABLES
+=back
 
-=head2 FULL_DEBUGGING
+=head1 TODO
 
-Setting this variable to 1 causes the C code to be compiled without
-optimization, allowing gdb to dump symbols of static functions with
-only one call site (which comprises most of the C code in
-L<Crypt::OpenSSL::CA>).  Also, the temporary build files are left
-intact if C<FULL_DEBUGGING> is set.
-
-Developpers, please note that in the absence of C<FULL_DEBUGGING>, the
-default compiler flags are C<-g -O2>, still allowing for a range of
-debugging strategies.  C<FULL_DEBUGGING> should therefore only be set
-on a one-shot basis by developpers who have a specific need for it.
-
-=head2 BUILD_OPENSSL_CFLAGS
-
-Contains the CFLAGS to pass so as to compile C code that links against
-OpenSSL; eg C<< -I/usr/lib/openssl/include >> or something.  Passed on
-to L<Inline::C/INC> by L</compile_everything>.
-
-=head2 BUILD_OPENSSL_LDFLAGS
-
-Contains the LDFLAGS to pass so as to link with the OpenSSL libraries;
-eg C<< -L/usr/lib/openssl/lib >> or something.  Passed on to
-L<Inline::C/LIBS> by L</compile_everything>.
-
-=head1 SEE ALSO
-
-L<Inline::C>, L<perlxstut>, L<perlguts>, L<perlapi>.
+Right now it is not possible to invoke
+I<Crypt::OpenSSL::CA::Inline::C> several times from the same package,
+yet allowing that would be straightforward.  the immediate benefit of
+that would be to allow to intermingle POD and C code freely.
 
 =cut
 
@@ -726,13 +471,11 @@ require My::Tests::Below unless caller();
 
 __END__
 
-=begin internals
-
 =head1 TEST SUITE
 
 =cut
 
-use Test::More "no_plan";
+use Test::More no_plan => 1;
 use Test::Group;
 use Crypt::OpenSSL::CA::Test qw(errstack_empty_ok
                                 cannot_check_SV_leaks leaks_SVs_ok
@@ -781,7 +524,7 @@ C_TEST
 
     my $object = make_bogus_object(42);
     is(ref($object), "bogoclass");
-    like($$object, qr/^-?[1-9][0-9]*$/, "looks like a number in the inside");
+    like($$object, qr/^[1-9][0-9]*$/, "looks like a number in the inside");
     is(bogus_object_value($object), 42);
     eval {
         $$object = 46;
@@ -808,8 +551,8 @@ char* TEST_char0_value(SV* scalar_under_test) {
 CHAR0_TEST
 
     is(Char0Test::TEST_char0_value(2 * 12), "24", "char0_value");
-    is(do { no warnings "uninitialized"; Char0Test::TEST_char0_value(undef) },
-       "", "char0_value shall not SEGV on undef");
+    is(Char0Test::TEST_char0_value(undef), "",
+       "char0_value shall not SEGV on undef");
 };
 
 skip_next_test "Devel::Leak needed" if cannot_check_SV_leaks;
@@ -839,60 +582,9 @@ C_TEST
     leaks_SVs_ok { map { Foo->new($_) } (1..1000) };
 };
 
-{ package TestCRoutines; use Crypt::OpenSSL::CA::Inline::C <<"C_TEST"; }
-void argh() {
-    sslcroak("How are you %s", "gentlemen");
-}
-
-// For leak tests
-void ulp() {
-    char buf[1024];
-    memset(buf, (int) 'A', 1023);
-    buf[1023] = '\\0';
-    sslcroak(buf);
-}
-
-int test_parse_RFC3280_time(char* time) {
-    char* plainerr = NULL; char* sslerr = NULL;
-    ASN1_TIME* asn1time;
-    int retval;
-
-    asn1time = parse_RFC3280_time(time, &plainerr, &sslerr);
-    if (asn1time) {
-        retval = (asn1time->type == V_ASN1_GENERALIZEDTIME ? 1 : 0);
-        ASN1_TIME_free(asn1time);
-        return retval;
-    }
-    if (plainerr) { return -1; }
-    if (sslerr) { return -2; }
-    return -42;
-}
-
-void test_parse_RFC3280_time_or_croak(char* time) {
-    ASN1_TIME_free(parse_RFC3280_time_or_croak(time));
-}
-
-int test_parse_serial(char* serial) {
-    char* plainerr = NULL; char* sslerr = NULL;
-    ASN1_INTEGER* asn1serial;
-
-    asn1serial = parse_serial(serial, &plainerr, &sslerr);
-    if (asn1serial) { ASN1_INTEGER_free(asn1serial); return 0; }
-    if (plainerr) { return 1; }
-    if (sslerr) { return 2; }
-    return 42;
-}
-
-void test_parse_serial_or_croak(char* serial) {
-    ASN1_INTEGER_free(parse_serial_or_croak(serial));
-}
-
-C_TEST
-
-
 test "sslcroak()" => sub {
     # Implementation lifted from Crypt::OpenSSL::CA so as
-    # to sever the circular dependency in tests:
+    # to sever the circular dependency in tests: 
     sub Crypt::OpenSSL::CA::_sslcroak_callback {
         my ($key, $val) = @_;
         if ($key eq "-message") {
@@ -910,8 +602,23 @@ test "sslcroak()" => sub {
         }
     }
 
+    { package TestSslcroak; use Crypt::OpenSSL::CA::Inline::C <<"C_TEST"; }
+void argh() {
+    sslcroak("How are you %s", "gentlemen");
+}
+
+// For leak tests
+void ulp() {
+    char buf[1024];
+    memset(buf, (int) 'A', 1023);
+    buf[1023] = '\\0';
+    sslcroak(buf);
+}
+
+C_TEST
+
     eval {
-        TestCRoutines::argh();
+        TestSslcroak::argh;
         fail("Should have thrown");
     };
     is(ref($@), "Crypt::OpenSSL::CA::Error") or warn Dumper($@);
@@ -921,71 +628,19 @@ test "sslcroak()" => sub {
     errstack_empty_ok();
     use Net::SSLeay;
     is(Net::SSLeay::BIO_new_file("/no/such/file_", "r"), 0);
-    eval { TestCRoutines::argh(); fail("should have thrown") };
+    eval { TestSslcroak::argh; fail("should have thrown") };
     is(ref($@), "Crypt::OpenSSL::CA::Error");
     is(ref($@->{-openssl}), "ARRAY");
     cmp_ok(scalar(@{$@->{-openssl}}), ">", 0);
-    my ($start_of_errors) = grep { $@->{-openssl}->[$_] =~ m/fopen/ }
-        (0..$#{$@->{-openssl}});
-    ok(defined($start_of_errors));
-    like($@->{-openssl}->[$start_of_errors + 1],
-         qr/no such file/); # Also checks that message wasn't truncated
+    # FIXME: absolute positions in stack probably not robust enough.
+    like($@->{-openssl}->[0], qr/fopen/);
+    like($@->{-openssl}->[1], qr/no such file/); # Also checks that
+    # message wasn't truncated
 
     unless (cannot_check_bytes_leaks) {
         leaks_bytes_ok {
-            for(1..200) { eval { TestCRoutines::ulp() }; }
+            for(1..200) { eval { TestSslcroak::ulp }; }
         };
     }
 };
 
-test "parse_RFC3280_time" => sub {
-    is(TestCRoutines::test_parse_RFC3280_time("20510103103442Z"), 1);
-    is(TestCRoutines::test_parse_RFC3280_time("19510103103442Z"), 0);
-    TestCRoutines::test_parse_RFC3280_time_or_croak("19510103103442Z");
-    eval {
-        TestCRoutines::test_parse_RFC3280_time_or_croak("0Z");
-        fail("Should have thrown");
-    };
-};
-
-skip_next_test "Devel::Mallinfo needed" if cannot_check_bytes_leaks;
-test "parse_RFC3280_time_or_croak memory leaks" => sub {
-    leaks_bytes_ok {
-        for(1..10000) {
-            eval {
-                TestCRoutines::test_parse_RFC3280_time_or_croak("portnawak");
-                fail("Should have thrown");
-            };
-            TestCRoutines::test_parse_RFC3280_time_or_croak("20510103103442Z");
-            TestCRoutines::test_parse_RFC3280_time_or_croak("19510103103442Z");
-        }
-    };
-};
-
-test "parse_serial" => sub {
-    TestCRoutines::test_parse_serial_or_croak("0xdeadbeef1234");
-    pass;
-    eval {
-        TestCRoutines::test_parse_serial_or_croak("deadbeef1234");
-        fail("should have thrown");
-    };
-    ok(! ref($@), "Plain error expected");
-    unlike($@, qr/unknown/i, "proper internal error management");
-    is(TestCRoutines::test_parse_serial("0xdeadbeef1234"), 0);
-    is(TestCRoutines::test_parse_serial("deadbeef1234"), 1);
-};
-
-skip_next_test "Devel::Mallinfo needed" if cannot_check_bytes_leaks;
-test "parse_serial memory leaks" => sub {
-    leaks_bytes_ok {
-        for(1..10000) {
-            TestCRoutines::test_parse_serial("0xdeadbeef1234");
-            TestCRoutines::test_parse_serial("deadbeef1234");
-        }
-    };
-};
-
-=end internals
-
-=cut
-
